/*****************************************************************************
* Author: Paulino Ruiz de Clavijo Vázquez <paulino@dte.us.es>
* Date: 05-12-2012
* Revision: 0.3
* Project: Nanofs
/** -- End auto header, don't touch this line -- ****************************/
 
#ifndef __LINUX_NANOFS_FS_H
#define __LINUX_NANOFS_FS_H



#define NANOFS_MAGIC           0x4e61
#define NANOFS_MAXFILENAME     255
#define NANOFS_DEFAULT_BLKSIZE 512
#define NANOFS_REVISION        0

// Nodes size in bytes
#define NANOFS_SB_SIZE sizeof(nanofs_super_block)
#define NANOFS_DIR_NODE_SIZE sizeof(nanofs_dir_node)
#define NANOFS_HDATA_NODE_SIZE 8 // Size of header

/* Flags for f_type field in directory node structure */
#define NANOFS_FLG_FTYPE  0 // bit 0: 1 for directory, 0 reg file

#define DN_ISREG(dn) (!(dn.d_flags & 0x01))  ///< Is dir node regular file?
#define DN_ISDIR(dn) (  dn.d_flags & 0x01 )  ///< Is dir node directory?


struct nanofs_superblock
{
    __u16 s_magic;
    __u8  s_blocksize;
    __u8  s_revision;
    __u32 s_alloc_ptr;
    __u32 s_free_ptr;
    __u32 s_fs_size;
    __u16 s_extra_size;
};

/* On disk nodes type */

/* Be carefully reading this struct from device
 * is not aligned to 8bits in memory. In disk is aligned to 8bits
 * */

struct nanofs_dir_node
{
    __u8  d_flags;
    __u32 d_next_ptr;
    __u32 d_data_ptr;
    __u32 d_meta_ptr;
    __u8  d_fname_len;
    __u8  d_fname[NANOFS_MAXFILENAME];
};

/* This struct is aligned */

struct nanofs_data_node
{
    __u32 d_next_ptr;
    __u32 d_len; // Data free length after this field in bytes
    void* d_data;
};

#endif
