/*****************************************************************************
* Author: Paulino Ruiz de Clavijo Vázquez <paulino@dte.us.es>
* Date: 05-12-2012
* Revision: 0.3
* Project: Nanofs
/** -- End auto header, don't touch this line -- ****************************/

#ifndef __NANOFS_FILEDIR__
#define __NANOFS_FILEDIR__

#include<nanofs.h>

#define NANOFS_NODETYPEDIR  0
#define NANOFS_NODETYPEDATA 1

#define NANOFS_MAX_PATH (NANOFS_MAXFILENAME+1)*100

#define NANOFSMGR_ERR_IO   1  ///< IO Error
#define NANOFSMGR_ERR_SB   2  ///< Superblock error
#define NANOFSMGR_ERR_DIR  3  ///< Dir structure error
#define NANOFSMGR_ERR_FREE 3  ///< Free space structure error



struct nanofs_fs_handle {
    char *h_dev_name;
    int  h_fd; // Open file descriptor
    int  h_block_bits;
    struct nanofs_superblock h_sb;
    __u32 h_blk_no; // Current dir block node number
    int h_error;
};

// This struct is filled with lookup func
struct nanofs_file_info {
    __u32 f_blk_no;             // Block number of dir entry
    __u32 f_prev_dir_blk_no;    // Prev dir node in list, 0 if first
    __u32 f_parent_dir_blk_no;  // Parent dir node
    long int f_size;
    struct nanofs_dir_node f_dir_node;
};

int nanofs_open_dev(char *dev,struct nanofs_fs_handle *handle);
int nanofs_close_dev(struct nanofs_fs_handle *handle);

int nanofs_free(struct nanofs_fs_handle *handle);

int nanofs_list_dir(struct nanofs_file_info fi_vec[],int vec_size,struct nanofs_fs_handle *handle);
int nanofs_lookup(char *file_name,struct nanofs_file_info *fi,struct nanofs_fs_handle *hd);

int nanofs_mkdir(char *dir_name,struct nanofs_fs_handle *handle);
int nanofs_rmdir(char *dir_name,struct nanofs_fs_handle *handle);
int nanofs_chdir(char *dir_name,struct nanofs_fs_handle *handle);
int nanofs_chpath(char *full_path,struct nanofs_fs_handle *hd);

int nanofs_get_file_info(char *file_name,struct nanofs_file_info *info,struct nanofs_fs_handle *handle);
int nanofs_put_file(char *local_file_name,char *dst_file_name,struct nanofs_fs_handle *handle);
int nanofs_get_file(char *local_file_name,char *file_name,struct nanofs_fs_handle *handle);
int nanofs_rm(char *file_name,struct nanofs_fs_handle *handle);

#endif
