/*****************************************************************************
* Author: Paulino Ruiz de Clavijo Vázquez <paulino@dte.us.es>
* Date: 05-12-2012
* Revision: 0.3
* Project: Nanofs
/** -- End auto header, don't touch this line -- ****************************/
 
#define _LARGEFILE64_SOURCE
#include <asm/types.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>

#include <nanofs.h>

/** Be carefully with memory aligment in C structs */
int nanofs_write_sb(int fd, off64_t offset, struct nanofs_superblock *sb)
{
    int res=nanofs_write_dev ( fd, offset,  sb, sizeof ( struct nanofs_superblock ));
    // Superblock struct is mem aligned, straightforward writing
    if(res!=0)
      fprintf (stderr,"** Writes superblock failed.\n" );
    
    return res;
}

int nanofs_read_sb(int fd, off64_t offset,struct nanofs_superblock *sb)
{
    if ( lseek64 ( fd, offset, SEEK_SET ) != offset )
    {
        fprintf(stderr,"** IO Error reading superblock, lseek failed: '%s'\n", strerror ( errno ));
        return -1;
    }
    if (read ( fd, sb,sizeof(struct nanofs_superblock) ) == sizeof(struct nanofs_superblock))
        return 0;
    else
        return -1;
}
/** Write dir node
* @return 0 on sucess
*/
int nanofs_write_dir_node(int fd, off64_t offset, struct nanofs_dir_node *dn)
{
    int err=0;
    off64_t c_off=offset;
    if (nanofs_write_dev ( fd, c_off,  &(dn->d_flags),1)!=0)
        err=-1;
    c_off++;
    if (err==0 && nanofs_write_dev ( fd, c_off,  &(dn->d_next_ptr),4)!=0)
        err=-1;;
    c_off+=4;
    if (err==0 && nanofs_write_dev ( fd, c_off,  &(dn->d_data_ptr),4)!=0)
        err=-1;;
    c_off+=4;
    if (err==0 && nanofs_write_dev ( fd, c_off,  &(dn->d_meta_ptr),4)!=0)
        err=-1;;
    c_off+=4;
    if (err==0 && nanofs_write_dev ( fd, c_off,  &(dn->d_fname_len),1)!=0)
        err=-1;;
    c_off+=1;
    if (err==0 && nanofs_write_dev ( fd, c_off,  &(dn->d_fname),NANOFS_MAXFILENAME)!=0)
        err=-1;;
    if(err!=0)
        fprintf (stderr,"** Error writting dir node on device, write error: '%s'.\n", strerror ( errno ) );
    return err;
}
int nanofs_write_data_block(int fd, off64_t offset, struct nanofs_data_node *db)
{
    int res=nanofs_write_dev ( fd, offset,  db,sizeof(struct nanofs_data_node)); 
    if(res!=0)
      fprintf (stderr,"** Data block writing failed.\n" );
    if(res!=0)
      
    return res;
}

/** Writes size bytes from offset
*  @return 0 on success, -1 on failure
*/
int nanofs_write_dev ( int fd, off64_t offset,const void *buf, int size )
{
    //printf(" -> nanofs_write_dev offset: %lxH\n",offset);
    //printf(" -> nanofs_write_dev size: %d\n",size);
    //printf(" -> nanofs_write_dev buf:\n");
    //dumphex(buf,size);
    //printf("\n");

    if ( lseek64 ( fd, offset, SEEK_SET ) != offset )
    {
        fprintf (stderr, "** Error writting data on device, 'lseek' fails: %s.\n", strerror ( errno ) );
        return -1;
    }
    if ( write ( fd, buf, size ) != size )
    {

        fprintf (stderr,"** Error writting  %d bytes on device, 'write' error: %s.\n", size,strerror ( errno ) );
        return -1;
    }
    return 0;
}


int nanofs_read_dir_node(int fd, off64_t offset,struct nanofs_dir_node *dn)
{
    if ( lseek64 ( fd, offset, SEEK_SET ) != offset )
        return -1;
    if (read(fd,&dn->d_flags,1)!=1)
        return -1;
    if (read(fd,&dn->d_next_ptr,4)!=4)
        return -1;
    if (read(fd,&dn->d_data_ptr,4)!=4)
        return -1;
    if (read(fd,&dn->d_meta_ptr,4)!=4)
        return -1;
    if (read(fd,&dn->d_fname_len,1)!=1)
    {
        if (dn->d_fname_len > NANOFS_MAXFILENAME)
            return -1;
    }
    if (read(fd,dn->d_fname,dn->d_fname_len)!=dn->d_fname_len)
        return -1;
    dn->d_fname[dn->d_fname_len]=0;
    return 0;
}

int nanofs_read_data_block(int fd, off64_t offset,struct nanofs_data_node *db)
{
    if ( lseek64 ( fd, offset, SEEK_SET ) != offset )
        return -1;
    if (read(fd,&db->d_next_ptr,4)!=4)
        return -1;
    if (read(fd,&db->d_len,4)!=4)
        return -1;
    return 0;
}
