/*****************************************************************************
* Author: Paulino Ruiz de Clavijo Vázquez <paulino@dte.us.es>
* Date: 05-12-2012
* Revision: 0.3
* Project: Nanofs
/** -- End auto header, don't touch this line -- ****************************/
 
 


#define _LARGEFILE64_SOURCE

#include<config.h>

#include <asm/types.h>
#include <sys/mount.h>
#include <mntent.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <libintl.h>
#include <locale.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <nanofs.h>
#include "utils.h"
#include "nanofs_io.h"

int dumpnanofs(char *device);


/** Help str */
static const char *HelpStr = "Usage: %s [OPTION...] file or device \n"\
                             "Options\n"\
                             "\t-c Dump file contents\n"\
                             "\t-V Version number\n"\
                             "\t-v Increase verbosity\n";

// Global options
int global_verbose=0; // Global verbosity
int global_file_contents=0; // Dump file contents



int main ( int argc, char **argv )
{
    int show_help=0;
    int optc;

    setlocale ( LC_ALL, "" );
    bindtextdomain ( "nanofs", "/usr/share/locale" );
    textdomain ( "nanofs" );
    print_version(basename(argv[0]));
    while ( ( optc = getopt ( argc, argv, "cb:v:V" ) ) != -1 )
    {
        switch ( optc )
        {
        case 'c':
            global_file_contents=1;
            break;
        case ':':
            printf ( "Argument missing, see usage.\n" );
        case '?':
        default:
            printf(HelpStr,argv[0]);
            return EXIT_FAILURE;
            break;
        case 'V':
            printf("dumpnanofs Version %s\n",VERSION);
            return EXIT_SUCCESS;
        case 'v':
            global_verbose = 1;
            break;
        }
    }
    dumpnanofs(argv[optind]);
    return EXIT_SUCCESS;
}


int dumpnanofs(char *device_str)
{
    int err=0,bytes;
    unsigned long long dev_size=1000000L;
    int fd;
    int blk_size=0;
    off64_t current_off=0;
    int blk_bits;
    int dn_blkno;
    char buf[1024];
    int i;

    struct nanofs_superblock sb;
    struct nanofs_dir_node dn;
    struct nanofs_data_node data_node;

    fd = open ( device_str, O_FSYNC  | O_RDONLY, 0 );
    if (fd<0)
    {
        fprintf(stderr,"** Error cannot open device %s\n",device_str);
        return -1;
    }

    dev_size=lseek64 ( fd, (off64_t)0, SEEK_END );
    printf("Opened device/file of size %lld bytes (%s)\n",dev_size,ltoh(dev_size));

    if (nanofs_read_sb(fd,(off64_t)0,&sb)!=0)
    {
        fprintf(stderr,"** Error cannot read superblock %s\n",device_str);
        err=-1;
    }
    // Dump superblock
    if (err==0)
    {
        printf("Superblock data:\n");
        printf(" - Magic number:        %4x", sb.s_magic);
        if (sb.s_magic!=NANOFS_MAGIC)
            printf(" [ERROR]\n");
        else
            printf(" [OK]\n");
        printf(" - Block size:        ");
        if (sb.s_blocksize=0)
        {
            blk_size=1;
            blk_bits=0;
        }
        else if (sb.s_blocksize=1)
        {
            blk_size=512;
            blk_bits=9;
        }
        else
            blk_size=-1;
        if (blk_size>0)
            printf(" %5d [OK]\n",blk_size);
        else
            printf(" %2x [ERROR]\n",sb.s_blocksize);
        printf(" - Filesystem revision: %4d", sb.s_revision);
        if (sb.s_revision!=0)
            printf(" [ERROR]\n");
        else
            printf(" [OK]\n");
        printf(" - Root on block:      0x%4x\n", sb.s_alloc_ptr);
        printf(" - Filesystem size:%9d blocks = ", sb.s_fs_size);
        printf(" %d Bytes (%s)\n", sb.s_fs_size*blk_size,ltoh(sb.s_fs_size*blk_size));
        printf(" - Free blocks at block:0x%4x", sb.s_free_ptr);
        if (sb.s_free_ptr >sb.s_fs_size)
        {
            printf("[ERROR] ** Beyond of device\n");
        }
        else
            printf("[OK]\n");


        printf(" - Superblock extrasize:%4d Bytes\n", sb.s_extra_size);

    }

    // Dump free blocks
     if (err==0)
      err=dump_free_blocks(fd,&sb,blk_bits);
    // Dump directories


    if (err==0)
    {
        printf("Root directory         ");
        if (nanofs_read_dir_node(fd,sb.s_alloc_ptr << blk_bits,&dn)!=0)
        {
            printf(" [READ Error]\n");
            err=-2;
        }
        else
            printf(" [Read OK]\n");
    }
    if (err==0)
    {
        if (DN_ISDIR(dn))
            printf(" - Entry type set as directory     [OK]\n");
        else
            printf(" - Entry type not set as directory [ERROR]\n");
        if (dn.d_next_ptr!=0)
            printf(" - Next entry on root dir          [ERROR]\n");
        else
            printf(" - Next entry on root dir          [OK]\n");
        printf(" - Root child dir entries at block: 0x%x\n",dn.d_data_ptr);
        printf(" - Volumen label: %s\n",dn.d_fname);
    }

    // Recursive dump dirs
    err=dump_directory(fd,blk_bits,sb.s_alloc_ptr,0);

    /*
    if (err==0 && dn.d_data_ptr!=0)
    {
        dn_blkno=dn.d_data_ptr; // First directory entry
        printf("Reading root directory\n");
        while(dn_blkno!=0)
        {
            if (nanofs_read_dir_node(fd,dn_blkno << blk_bits,&dn)!=0)
            {
            printf("**Error reading directory entry\n");
            err=-3;
            break;
            }
            printf(" - Directory node at block %xH (offset %xH)\n",dn_blkno,dn_blkno << blk_bits);
            if(DN_ISREG(dn))
                printf("   + Node type (flag):       FILE\n");
            if(DN_ISDIR(dn))
                printf("   + Node type (flag):       DIR\n");
            printf("   + Next dir node at block: %xH\n",dn.d_next_ptr);
            printf("   + Metadata at block:      %xH\n",dn.d_meta_ptr);
            printf("   + FName length:           %-d\n",dn.d_fname_len);
            printf("   + FName:                  '%s'\n",dn.d_fname);
            printf("   + Data start at block:    %xH\n",dn.d_data_ptr);

            // Data blocks
            if(dn.d_data_ptr!=0)
            {
                if( nanofs_read_data_block(fd,dn.d_data_ptr << blk_bits,&data_node) != 0)
                {
                    fprintf(stderr,"** Error reading data block\n");
                    err=-__LINE__;
                    break;
                }
                printf("     > Data block next:  %xH\n",data_node.d_next_ptr);
                printf("     > Data block len:   %d Bytes\n",data_node.d_len);

                bytes=read(fd,buf,1024);
                for(i=0;i<data_node.d_len;i++)
                {
                    if(i%64==0)
                        printf("\n         ");
                    if(buf[i % 1024]>32 && buf[i % 1024]<126)
                        printf("%c",buf[i % 1024]);
                    else
                        printf(".");
                    bytes--;
                    if(bytes==0)
                        bytes=read(fd,buf,1024);
                }
                printf("\n");
            }
            dn_blkno=dn.d_next_ptr;
        }
    }
    */
    close(fd);
    return err;
}

/** format utils */
void print_tabs(int n)
{
    int t;
    for (t=0;t<n;t++)
        printf("    ");
}

/** Recursive dump directory
 * @return number of errors found, 0 on success */
int dump_directory(int fd_dev,int blk_bits,int dir_blkno,int level)
{
    struct nanofs_dir_node dir_node;
    struct nanofs_data_node data_node;
    int current_blkno;
    int err=0;
    print_tabs(level);
    printf(" - Dump directory on level %d,",level);
    if (nanofs_read_dir_node(fd_dev,(off64_t)(dir_blkno) << blk_bits,&dir_node)!=0)
    {
        printf(" [READ Error]\n");
        return 1;
    }
    printf(" name '%s' [OK]\n",dir_node.d_fname);

    print_tabs(level);
    printf(" - Directory entry at block 0x%x (offset 0x%x), data:\n",dir_blkno,dir_blkno << blk_bits);

    if (DN_ISDIR(dir_node))
    {
        print_tabs(level);
        printf("   + Node type (flag):      DIR [OK]\n");
    }
    else
    {
        print_tabs(level);
        printf("   + Node type (flag):      REG_FILE [ERROR]\n");
        err++;
    }
    print_tabs(level);
    printf("   + Next dir node at block: 0x%x\n",dir_node.d_next_ptr);
    print_tabs(level);
    printf("   + Metadata at block:      0x%x\n",dir_node.d_meta_ptr);
    print_tabs(level);
    printf("   + FName length:            %-d\n",dir_node.d_fname_len);
    print_tabs(level);
    printf("   + FName:                  '%s'\n",dir_node.d_fname);
    print_tabs(level);
    printf("   + Child nodes at block:   0x%x\n",dir_node.d_data_ptr);

    if (err)
        return err;

    // Listing dir contents
    current_blkno=dir_node.d_data_ptr; // First directory entry
    while (current_blkno!=0)
    {

        if (nanofs_read_dir_node(fd_dev,(off64_t)(current_blkno) << blk_bits,&dir_node)!=0)
        {
            printf("** [Err:%d] Error reading directory entry\n",__LINE__);
            return ++err;
        }

        if (DN_ISDIR(dir_node))
        {
            // Recursive call
            err+=dump_directory(fd_dev,blk_bits,current_blkno,level+1);
        }
        else if (DN_ISREG(dir_node))
        {
            print_tabs(level+1);
            printf(" - Directory entry at block 0x%x (offset 0x%x), data:\n",current_blkno,current_blkno << blk_bits);
            print_tabs(level+1);
            printf("   + Node type (flag):       REG_FILE\n");
            print_tabs(level+1);
            printf("   + Next dir node at block: 0x%x\n",dir_node.d_next_ptr);
            print_tabs(level+1);
            printf("   + Metadata at block:      0x%x\n",dir_node.d_meta_ptr);
            print_tabs(level+1);
            printf("   + FName length:            %-d\n",dir_node.d_fname_len);
            print_tabs(level+1);
            printf("   + FName:                  '%s'\n",dir_node.d_fname);
            print_tabs(level+1);
            printf("   + Data start at block:    0x%x\n",dir_node.d_data_ptr);
            // Data blocks
            if (dir_node.d_data_ptr!=0)
            {
                if ( nanofs_read_data_block(fd_dev,(off64_t)(dir_node.d_data_ptr) << blk_bits,&data_node) != 0)
                {
                    fprintf(stderr,"** [Err %d] Error reading data block\n",__LINE__);
                    return ++err;
                }
                print_tabs(level+1);
                printf("     > Data block next:  0x%x\n",data_node.d_next_ptr);
                print_tabs(level+1);
                printf("     > Data block len:    %d Bytes (%s)\n",data_node.d_len,ltoh(data_node.d_len));
                if (global_file_contents)
                    err+=dump_file_contents(fd_dev,blk_bits,dir_node.d_data_ptr);
            }
        }
        //if (DN_ISDIR(dir_node))


        // Data blocks
        /*if (dir_node.d_data_ptr!=0)
        {
            if ( nanofs_read_data_block(fd_dev,(off64_t)(dir_node.d_data_ptr) << blk_bits,&data_node) != 0)
            {
                fprintf(stderr,"** [Err %d] Error reading data block\n",__LINE__);
                return ++err;
            }
            print_tabs(level+1);
            printf("     > Data block next:  %xH\n",data_node.d_next_ptr);
            print_tabs(level+1);
            printf("     > Data block len:   %d Bytes\n",data_node.d_len);
            if (global_file_contents)
                err+=dump_file_contents(fd_dev,blk_bits,dir_node.d_data_ptr);

             }*/
        current_blkno=dir_node.d_next_ptr;
    }
}

int dump_file_contents(int fd_dev,int blk_bits,int data_blkno)
{
    char buf[1024];
    int i;
    int err=0;
    struct nanofs_data_node data_nd;
    int bytes;
    if ( nanofs_read_data_block(fd_dev,(off64_t)(data_blkno) << blk_bits,&data_nd) != 0)
    {
        fprintf(stderr,"** [Err %d] Error reading data block\n",__LINE__);
        return ++err;
    }

    bytes=read(fd_dev,buf,1024);
    for (i=0;i<data_nd.d_len;i++)
    {
        if (i%64==0)
            printf("\n         ");
        if (buf[i % 1024]>32 && buf[i % 1024]<126)
            printf("%c",buf[i % 1024]);
        else
            printf(".");
        bytes--;
        if (bytes==0)
            bytes=read(fd_dev,buf,1024);
    }
    printf("\n");
    return err;
}


int dump_free_blocks(int fd,struct nanofs_superblock *sb,int blk_bits)
{
    int err=0;
    struct nanofs_data_node free_node;
    __u32 node_no;
    node_no=sb->s_free_ptr;
    printf("Reading free blocks\n");
       printf(" - First free block at block 0x%x (offset 0x%x), dump legend (blk_no,size):\n",node_no,node_no << blk_bits);
       printf("   + ");
	while(node_no > 0)	  
	{
	    if ( nanofs_read_data_block(fd,node_no << blk_bits,&free_node) != 0)
	    {
	      err=-__LINE__;
	      fprintf(stderr,"** [Err:%d] IO Error reading free block\n",-err);
	      break;
	    }
	  
	  printf("(0x%x,%d)",node_no,free_node.d_len);
	  node_no=free_node.d_next_ptr;
	  if(node_no>0)
	    printf("->");
	}
	printf("\n");
  return err;
}