#!/bin/bash
echo "### Start test 3: Testing nanofs-manager with directories"

MANAGER=../progs/nanofs-manager
MKFS=../progs/mkfs.nanofs

fail() {
    echo "**** Test Failed !!"
    exit 255
}

mkdir_level_1()
{
  $MANAGER nanofs.raw mkdir /level1_dir1 || fail
  $MANAGER nanofs.raw mkdir /level1_dir2 || fail
  $MANAGER nanofs.raw mkdir /level1_dir3 || fail
}

if [ ! -f nanofs.raw ] ; then
  echo "## Creating empty file to test filesystem (1MiB)"
  dd if=/dev/zero of=nanofs.raw bs=1M count=1 > /dev/null
else
  echo "## Using file nanofs.raw"
fi
echo "## Testing dirs on level 1"

$MKFS nanofs.raw 
mkdir_level_1
$MANAGER nanofs.raw rmdir /level1_dir1 || fail
$MANAGER nanofs.raw rmdir /level1_dir2 || fail
$MANAGER nanofs.raw rmdir /level1_dir3 || fail

echo "## Level 1 [OK]"
echo "## Testing multilevel dirs"
$MKFS nanofs.raw 

$MANAGER nanofs.raw mkdir /level1 || fail
$MANAGER nanofs.raw mkdir /level1/level2 || fail
$MANAGER nanofs.raw mkdir /level1/level2/level3 || fail
$MANAGER nanofs.raw mkdir /level1/level2/level3/level4 || fail

$MANAGER nanofs.raw ls    /level1/level2/level3/level4 || fail
$MANAGER nanofs.raw ls    /level1/level2/level3 || fail
$MANAGER nanofs.raw ls    /level1/level2 || fail
$MANAGER nanofs.raw ls    /level1 || fail
$MANAGER nanofs.raw ls    / || fail

$MANAGER nanofs.raw rmdir  /level1/level2/level3/level4 || fail
$MANAGER nanofs.raw ls    /level1/level2/level3 || fail
$MANAGER nanofs.raw ls    /level1/level2 || fail
$MANAGER nanofs.raw ls    /level1 || fail
$MANAGER nanofs.raw ls    / || fail

$MANAGER nanofs.raw rmdir  /level1/level2/level3 || fail
$MANAGER nanofs.raw ls    /level1/level2 || fail
$MANAGER nanofs.raw ls    /level1 || fail
$MANAGER nanofs.raw ls    / || fail

$MANAGER nanofs.raw rmdir  /level1/level2 || fail
$MANAGER nanofs.raw ls    /level1 || fail
$MANAGER nanofs.raw ls    / || fail

$MANAGER nanofs.raw rmdir  /level1 || fail
$MANAGER nanofs.raw ls    / || fail
AC_RES=`$MANAGER nanofs.raw free`
echo "## Testing dirs on level 2"
$MKFS nanofs.raw
FREESP1=`$MANAGER nanofs.raw free`
 
mkdir_level_1
$MANAGER nanofs.raw mkdir /level1_dir2/level2_dir1 || fail
$MANAGER nanofs.raw mkdir /level1_dir3/level2_dir1 || fail
$MANAGER nanofs.raw mkdir /level1_dir3/level2_dir2 || fail
$MANAGER nanofs.raw mkdir /level1_dir3/level2_dir3 || fail

$MANAGER nanofs.raw rmdir /level1_dir2/level2_dir1 || fail

$MANAGER nanofs.raw rmdir /level1_dir2 || fail
$MANAGER nanofs.raw rmdir /level1_dir1 || fail
$MANAGER nanofs.raw rmdir /level1_dir3/level2_dir2 || fail
$MANAGER nanofs.raw rmdir /level1_dir3/level2_dir3 || fail
$MANAGER nanofs.raw rmdir /level1_dir3/level2_dir1 || fail
$MANAGER nanofs.raw rmdir /level1_dir3 || fail

FREESP2=`$MANAGER nanofs.raw free`

echo "Free space after and before $FREESP2 != $FREESP1"
echo "## Level 2 [OK]"
